""" Various shared code used in the scripts that leverage the mojo integration test container.
"""
import subprocess
import sys
import time

class Step(object):
    """ A simple context manager for tracking success and timing of a step"""
    def __init__(self, name, parent):
        """ Init the context manager
        :param name: The name of this object
        :param parent: An appendable data structure this object will be added to.
        """
        self.name = name
        self.success = True
        self.start = None
        self.stop = None
        self.time = None
        parent.append(self)

    def __enter__(self):
        self.start = time.time()
        return self

    def __exit__(self, type, value, traceback):
        self.end = time.time()
        self.time = self.end - self.start
        if isinstance(value, Exception):
            self.success = False


def pretty_time(seconds):
    """ Take a time in seconds and print out a human readable string, ie 5 minutes 3 seconds
    """
    if seconds > 60:
        return '{} minutes {:.3g} seconds'.format(int(seconds/60), seconds%60)
    else:
        return '{:.3g} seconds'.format(seconds)


def verify_prereqs():
    """Make sure Juju is connected to a controller.

    """
    try:
        subprocess.check_output(['juju', 'switch'])
    except subprocess.CalledProcessError, OSError:
        print('Error checking active Juju controller')
        sys.exit(1)
