# Copyright 2014 Canonical Ltd.  This software is licensed under the
# GNU General Public License version 3 (see the file LICENSE).
from __future__ import print_function
from mojo.cli import MojoArgumentParser
from mojo.juju.status import Status
import argparse
import logging
import sys


def main():
    description = """
    List properties for objects in a Juju environment.
    Available properties:

    service
       servicename
         ips
         instance_ids
         machine_numbers
         units
    machine
       machine_number
         ips
         instance_id

    e.g.:
        mojo-info service apache2 units
        mojo-info machine 1 instance_id"""
    ap = MojoArgumentParser(description=description,
                            formatter_class=argparse.RawTextHelpFormatter)

    ap.add_argument("type", nargs=1, help="Object type (unit, machine)")
    ap.add_argument("inst", nargs=1, help="Object instance")
    ap.add_argument("prop", nargs=1, help="Property to list")

    args = ap.parse_args()

    objtype = args.type[0]
    inst = args.inst[0]
    prop = args.prop[0]

    attr = "{}_{}".format(objtype, prop)
    status = Status()
    try:
        handler = getattr(status, attr)
    except AttributeError:
        ap.print_help()
        sys.exit(1)
    if not handler:
        logging.error("Can't get {} for a {}".format(prop, objtype))
    values = handler(inst)

    if values is None:
        logging.error("{} not found: {}".format(objtype.capitalize(), inst))
        sys.exit(1)
    elif not values:
        logging.warning("{} {} has no {}".format(objtype.capitalize(), inst,
                        prop))
    else:
        if isinstance(values, list):
            map(print, values)
        else:
            print(values)
    sys.exit(0)
