""" Provides a variety of check commands that can be run against Juju status.
    The output and exit code matches that expected by Nagios for Okay (0) or Critical (2).
    Juju version 2 only.
"""
import argparse
import inspect
import sys

import checks
from status import Juju2Status, JujuStatusError


def main():

    parser = argparse.ArgumentParser(description="Runs a variety of checks on Juju2 status.")
    parser.add_argument("-f", "--file", dest="status_file", help="A yaml status file, if unspecified status is" +
                                                                 "pulled from the environment.")

    sp = parser.add_subparsers(title="checks", dest="check_name")

    available_checks = {}
    for _, clsmember in inspect.getmembers(checks, inspect.isclass):
        if checks.JujuCheck == clsmember:
            continue
        if issubclass(clsmember, checks.JujuCheck):
            available_checks[clsmember.cmd_name] = clsmember

    for check in available_checks.values():
        sub = sp.add_parser(check.cmd_name, description=check.help)
        check.setup_argparse(sub)

    args = parser.parse_args()

    try:
        if args.status_file is not None:
            with open(args.status_file, 'r') as sf:
                status_yaml = sf.read()
            status = Juju2Status(status_yaml=status_yaml)
        else:
            status = Juju2Status()

        output = available_checks[args.check_name](status).check(args)
    except (JujuStatusError, checks.JujuCheckError, IOError) as e:
        print("CRITICAL: {}".format(e))
        return 2

    print("OK: {}.".format(output))
    return 0


if __name__ == "__main__":
    sys.exit(main())
