# Copyright 2015 Canonical Ltd.  This software is licensed under the
# GNU General Public License version 3 (see the file LICENSE).
import os
import cli
from .utils import setup_logging


def main():
    description = """
    mojo wrapper for creating a new Mojo project and container with sudo.
    This should only be used directly from automated scripts that cannot enter
    a sudo password.  Otherwise mojo will handle calling it when it needs to
    create a new project."""
    # Note this will only work with sudo if it is in the sudoers secure_path,
    # i.e. /usr/bin or /usr/local/bin

    series_args = ("-s", "--series")
    series_kwargs = {'default': os.environ.get('MOJO_SERIES'),
                     'help': "Distro series. Overrides the MOJO_SERIES environment variable."}

    ap = cli.MojoArgumentParser(description=description)

    ap.add_argument("-r", "--mojo_root", default=os.environ.get('MOJO_ROOT'),
                    help="Root directory that mojo works out of Overrides the MOJO_ROOT environment variable."
                         "Defaults to /srv/mojo for projects using 'lxc' container or containerless, "
                         "Defaults to ~/.local/share/mojo for lxd containers.")
    ap.add_argument(*series_args, required=True, **series_kwargs)
    ap.add_argument("name",
                    help="Base name of the project ([a-zA-Z0-9._]*)")
    ap.add_argument('-c', "--container", default=os.environ.get('MOJO_CONTAINER', 'lxc'),
                    choices=['lxc', 'lxd', 'containerless'],
                    help="Container. Options 'lxc', 'lxd' or 'containerless'. "
                         "Defaults to 'lxc'")
    ap.add_argument("--log_level", "-l", default='INFO',
                    help="Log level. Defaults to INFO")

    args = ap.parse_args()
    setup_logging(args.log_level)
    cli.create_project(args)
