# Copyright 2014 Canonical Ltd.  This software is licensed under the
# GNU General Public License version 3 (see the file LICENSE).
import mock
import subprocess
from unittest import TestCase

from mojo.juju.debuglogs import DebugLogs, UnsupportedDebugLogType, DEFAULT_DEBUG_LOG_CONFIG
from mojo.tests import utils


class JujuDebugLogsTestCase(TestCase, utils.MojoTestCaseMixin):

    def test__load_config_production_config(self):
        """Test _load_config does nothing with production stage"""
        ws = None
        debug_logs = DebugLogs(ws, "production", None, None)
        self.assertFalse(debug_logs._load_config())

    @mock.patch('subprocess.check_output')
    def test__resolve_remote_paths(self, _check_output):
        """Test _resolve_remote_paths"""
        _check_output.return_value = u"['/etc/init.d/apache2']"
        ws = None
        debug_logs = DebugLogs(ws, "production", None, None)
        self.assertEqual(debug_logs._resolve_remote_paths('apache2/0', '/etc/init.d/apache2'),
                         ['/etc/init.d/apache2'])
        _check_output.assert_called_with(
            ['juju', 'run', '--unit', 'apache2/0',
             'PYTHON=$(which python3 python | head -n1) ; [ -z "$PYTHON" ] && { echo ERROR: No python3 or python found. '
             '; exit 1 ; } ; $PYTHON -c "import glob; print(glob.glob(\'/etc/init.d/apache2\'))"'],
            stderr=subprocess.STDOUT)

    @mock.patch('subprocess.check_output')
    def test__run_remote_command(self, _check_output):
        """Test _run_remote_command"""
        ws = None
        debug_logs = DebugLogs(ws, "production", None, None)
        debug_logs._run_remote_command('apache2/0', 'cat', ['/etc/init.d/apache2'])
        _check_output.assert_called_with(
            ['juju', 'run', '--unit', 'apache2/0', 'cat /etc/init.d/apache2'],
            stderr=subprocess.STDOUT)

    def test__check_config(self):
        """Test _check_config"""
        ws = None
        debug_logs = DebugLogs(ws, "production", None, None)
        with mock.patch('mojo.juju.debuglogs.DebugLogs._load_config') as _load_config:
            _load_config.return_value = DEFAULT_DEBUG_LOG_CONFIG
            # Assert this doesn't raise an error
            debug_logs._check_config()
            # Now set the return value to something that will raise an error.
            # First of all, reset _config so that _load_config() will be triggered again.
            debug_logs._config = None
            _load_config.return_value = {
                'apache2': {
                    'bogus-debuglog-type': ['/etc/apache2']
                    }
                }
            with self.assertRaises(UnsupportedDebugLogType):
                debug_logs._check_config()
