# Copyright 2014 Canonical Ltd.  This software is licensed under the
# GNU General Public License version 3 (see the file LICENSE).
import mock
from unittest import TestCase

import mojo.juju
from mojo.tests import utils


NO_ERROR_STATUS = """environment: openstack
machines:
  "0":
    agent-state: started
    agent-version: 1.25.0
    dns-name: 10.55.33.5
    instance-id: d9824814-d4d7-4ffa-b600-24f6fecc615c
    instance-state: ACTIVE
    series: trusty
    hardware: arch=amd64 cpu-cores=1 mem=1024M root-disk=10240M availability-zone=nova
    state-server-member-status: has-vote
  "1":
    agent-state: started
    agent-version: 1.25.0
    dns-name: 10.55.33.6
    instance-id: 0a598fda-78b0-47dd-aa21-ef9ec9e6879b
    instance-state: ACTIVE
    series: trusty
    hardware: arch=amd64 cpu-cores=1 mem=1024M root-disk=10240M availability-zone=nova
services:
  apache2:
    charm: local:trusty/apache2-0
    exposed: true
    service-status:
      current: unknown
      since: 15 Dec 2015 10:12:55Z
    relations:
      juju-info:
      - content-fetcher
      nrpe-external-master:
      - nrpe
    units:
      apache2/0:
        workload-status:
          current: unknown
          since: 15 Dec 2015 10:12:55Z
        agent-status:
          current: idle
          since: 15 Dec 2015 10:17:59Z
          version: 1.25.0
        agent-state: started
        agent-version: 1.25.0
        machine: "1"
        open-ports:
        - 80/tcp
        public-address: 10.55.33.6
        subordinates:
          content-fetcher/0:
            workload-status:
              current: unknown
              since: 15 Dec 2015 10:16:01Z
            agent-status:
              current: idle
              since: 15 Dec 2015 10:17:41Z
              version: 1.25.0
            agent-state: started
            agent-version: 1.25.0
            upgrading-from: local:trusty/content-fetcher-0
            public-address: 10.55.33.6
          nrpe/0:
            workload-status:
              current: unknown
              since: 15 Dec 2015 10:16:42Z
            agent-status:
              current: idle
              since: 15 Dec 2015 10:18:00Z
              version: 1.25.0
            agent-state: started
            agent-version: 1.25.0
            upgrading-from: local:trusty/nrpe-external-master-7
            public-address: 10.55.33.6
  content-fetcher:
    charm: local:trusty/content-fetcher-0
    exposed: false
    service-status: {}
    relations:
      general-info:
      - apache2
    subordinate-to:
    - apache2
  nrpe:
    charm: local:trusty/nrpe-external-master-7
    exposed: false
    service-status: {}
    relations:
      nrpe-external-master:
      - apache2
    subordinate-to:
    - apache2"""

ERROR_STATUS = """environment: openstack
machines:
  "0":
    agent-state: started
    agent-version: 1.25.0
    dns-name: 10.55.33.5
    instance-id: d9824814-d4d7-4ffa-b600-24f6fecc615c
    instance-state: ACTIVE
    series: trusty
    hardware: arch=amd64 cpu-cores=1 mem=1024M root-disk=10240M availability-zone=nova
    state-server-member-status: has-vote
  "1":
    agent-state: started
    agent-version: 1.25.0
    dns-name: 10.55.33.6
    instance-id: 0a598fda-78b0-47dd-aa21-ef9ec9e6879b
    instance-state: ACTIVE
    series: trusty
    hardware: arch=amd64 cpu-cores=1 mem=1024M root-disk=10240M availability-zone=nova
services:
  apache2:
    charm: local:trusty/apache2-0
    exposed: true
    service-status:
      current: error
      message: 'hook failed: "config-changed"'
      since: 15 Dec 2015 10:24:24Z
    relations:
      juju-info:
      - content-fetcher
      nrpe-external-master:
      - nrpe
    units:
      apache2/0:
        workload-status:
          current: error
          message: 'hook failed: "config-changed"'
          since: 15 Dec 2015 10:24:24Z
        agent-status:
          current: idle
          since: 15 Dec 2015 10:24:24Z
          version: 1.25.0
        agent-state: error
        agent-state-info: 'hook failed: "config-changed"'
        agent-version: 1.25.0
        machine: "1"
        open-ports:
        - 80/tcp
        public-address: 10.55.33.6
        subordinates:
          content-fetcher/0:
            workload-status:
              current: unknown
              since: 15 Dec 2015 10:16:01Z
            agent-status:
              current: idle
              since: 16 Dec 2015 08:44:08Z
              version: 1.25.0
            agent-state: started
            agent-version: 1.25.0
            upgrading-from: local:trusty/content-fetcher-0
            public-address: 10.55.33.6
          nrpe/0:
            workload-status:
              current: unknown
              since: 15 Dec 2015 10:16:42Z
            agent-status:
              current: idle
              since: 16 Dec 2015 08:44:34Z
              version: 1.25.0
            agent-state: started
            agent-version: 1.25.0
            upgrading-from: local:trusty/nrpe-external-master-7
            public-address: 10.55.33.6
  content-fetcher:
    charm: local:trusty/content-fetcher-0
    exposed: false
    service-status: {}
    relations:
      general-info:
      - apache2
    subordinate-to:
    - apache2
  nrpe:
    charm: local:trusty/nrpe-external-master-7
    exposed: false
    service-status: {}
    relations:
      nrpe-external-master:
      - apache2
    subordinate-to:
    - apache2"""


class Juju1StatusTestCase(TestCase, utils.MojoTestCaseMixin):

    def setUp(self):
        self.status = mojo.juju.status.Juju1Status()
        self.status._status_output = NO_ERROR_STATUS

    def test_status(self):
        """Test the status function"""
        self.assertEqual(self.status.status(), self.status._status_output)

    def test__get_status(self):
        """Test that _get_status is called with the right args"""
        with mock.patch('subprocess.check_output') as mock_check_output:
            # First check with no environment set
            no_env_status = mojo.juju.status.Juju1Status()
            no_env_status._get_status()
            mock_check_output.assert_called_with(
                ['/usr/bin/timeout', '--kill-after', '5', '600',
                 'juju', 'status', '--format=yaml'])
            # Second test with an environment set
            env_status = mojo.juju.status.Juju1Status(environment="test-env")
            env_status._get_status()
            mock_check_output.assert_called_with(
                ['/usr/bin/timeout', '--kill-after', '5', '600',
                 'juju', 'status', '--format=yaml', '-e', 'test-env'])

        with mock.patch('subprocess.check_output') as mock_check_output:
            # Check that if we're calling _get_status where _status_output is
            # set, it doesn't query juju status
            self.assertTrue(self.status._status_output)
            self.status._get_status()
            self.assertFalse(mock_check_output.called)
            # And now check if we pass the force_update option, that it is
            # called
            self.status._get_status(force_update=True)
            mock_check_output.assert_called_with(
                ['/usr/bin/timeout', '--kill-after', '5', '600',
                 'juju', 'status', '--format=yaml'])

    def test_yaml_status(self):
        """Test the yaml_status function"""
        self.assertEqual(self.status.yaml_status(), {
            'environment': 'openstack', 'services': {
                'nrpe': {
                    'service-status': {},
                    'charm': 'local:trusty/nrpe-external-master-7',
                    'relations': {
                        'nrpe-external-master': ['apache2']
                        },
                    'subordinate-to': ['apache2'],
                    'exposed': False
                    },
                'content-fetcher': {
                    'service-status': {},
                    'charm': 'local:trusty/content-fetcher-0',
                    'relations': {
                        'general-info': ['apache2']
                        },
                    'subordinate-to': ['apache2'],
                    'exposed': False
                    },
                'apache2': {
                    'units': {
                        'apache2/0': {
                            'public-address': '10.55.33.6',
                            'workload-status': {
                                'current': 'unknown',
                                'since': '15 Dec 2015 10:12:55Z'
                                },
                            'agent-status': {
                                'current': 'idle',
                                'since': '15 Dec 2015 10:17:59Z',
                                'version': '1.25.0'
                                },
                            'agent-state': 'started',
                            'subordinates': {
                                'content-fetcher/0': {
                                    'upgrading-from': 'local:trusty/content-fetcher-0',
                                    'public-address': '10.55.33.6',
                                    'workload-status': {
                                        'current': 'unknown',
                                        'since': '15 Dec 2015 10:16:01Z'
                                        },
                                    'agent-status': {
                                        'current': 'idle',
                                        'since': '15 Dec 2015 10:17:41Z',
                                        'version': '1.25.0'
                                        },
                                    'agent-state': 'started',
                                    'agent-version': '1.25.0'
                                    },
                                'nrpe/0': {
                                    'upgrading-from': 'local:trusty/nrpe-external-master-7',
                                    'public-address': '10.55.33.6',
                                    'workload-status': {
                                        'current': 'unknown',
                                        'since': '15 Dec 2015 10:16:42Z'
                                        },
                                    'agent-status': {
                                        'current': 'idle',
                                        'since': '15 Dec 2015 10:18:00Z',
                                        'version': '1.25.0'
                                        },
                                    'agent-state': 'started',
                                    'agent-version': '1.25.0'
                                    }
                                },
                            'machine': '1',
                            'open-ports': ['80/tcp'],
                            'agent-version': '1.25.0'
                            }
                        },
                    'service-status': {
                        'current': 'unknown',
                        'since': '15 Dec 2015 10:12:55Z'
                        },
                    'charm': 'local:trusty/apache2-0',
                    'relations': {
                        'juju-info': ['content-fetcher'],
                        'nrpe-external-master': ['nrpe']
                        },
                    'exposed': True
                    }
                },
            'machines': {
                '1': {
                    'hardware': 'arch=amd64 cpu-cores=1 mem=1024M root-disk=10240M availability-zone=nova',
                    'instance-state': 'ACTIVE',
                    'agent-version': '1.25.0',
                    'agent-state': 'started',
                    'series': 'trusty',
                    'instance-id': '0a598fda-78b0-47dd-aa21-ef9ec9e6879b',
                    'dns-name': '10.55.33.6'
                    },
                '0': {
                    'instance-state': 'ACTIVE',
                    'state-server-member-status': 'has-vote',
                    'agent-state': 'started',
                    'series': 'trusty',
                    'dns-name': '10.55.33.5',
                    'hardware': 'arch=amd64 cpu-cores=1 mem=1024M root-disk=10240M availability-zone=nova',
                    'agent-version': '1.25.0',
                    'instance-id': 'd9824814-d4d7-4ffa-b600-24f6fecc615c'
                    }
                }
            })

    def test__get_services_list(self):
        """"Test getting services_list"""
        self.assertEqual(self.status.services_list(), ['apache2'])
        self.assertEqual(self.status.services_list(exclude_subordinates=False), ['apache2', 'content-fetcher', 'nrpe'])

    def test__get_machine_ids_list(self):
        """Test getting all machine IDs"""
        self.assertEqual(self.status.machine_ids_list(), ['0', '1'])

    def test__get_machine_agent_states(self):
        """Test all machine agents started"""
        self.assertEqual(self.status._get_machine_agent_states(),
                         {'started': 2})

    def test__get_machine_instance_states(self):
        """Test all machine instances ACTIVE"""
        self.assertEqual(self.status._get_machine_instance_states(),
                         {'ACTIVE': 2})

    def test__get_units_in_state(self):
        """Confirm no units in error, all units started"""
        self.assertEqual(self.status._get_units_in_state("error"), [])
        self.assertEqual(self.status._get_units_in_state("started"),
                         ['apache2/0', 'content-fetcher/0', 'nrpe/0'])

    @mock.patch('mojo.juju.status.Juju1Status._get_status')
    @mock.patch('subprocess.check_output')
    @mock.patch('time.sleep')
    def test_check_and_wait(self, _sleep, _check_output, _get_status):
        """Test our environment isn't in error"""
        # Confirm we don't raise a JujuStatusError
        _get_status.return_value = NO_ERROR_STATUS
        self.status.check_and_wait()
        # And now confirm we do raise JujuStatusError for an environment in
        # error state
        _get_status.return_value = ERROR_STATUS
        with self.assertRaises(mojo.juju.JujuStatusError):
            self.status.check_and_wait()

    def test__all_stable(self):
        """Confirm returns True unless any state is transitional"""
        self.assertTrue(self.status._all_stable({'started': 2}))
        self.assertTrue(self.status._all_stable({'error': 2}))
        self.assertTrue(self.status._all_stable({'started': 1, 'error': 1}))
        self.assertFalse(self.status._all_stable({'started': 1, 'pending': 1}))

    def test__error_check(self):
        """Confirm returns False unless any state is in error"""
        self.assertFalse(self.status._error_check({'started': 2}))
        self.assertTrue(self.status._error_check({'error': 2}))
        self.assertTrue(self.status._error_check({'started': 1, 'error': 1}))
        self.assertFalse(self.status._error_check({'started': 1, 'pending': 1}))

    def test_service_ips(self):
        self.assertEqual(self.status.service_ips('apache2'), ['10.55.33.6'])

    def test_service_units(self):
        self.assertEqual(self.status.service_units('apache2'), ['apache2/0'])

    def test_service_machine_numbers(self):
        self.assertEqual(self.status.service_machine_numbers('apache2'), ['1'])

    def test_service_instance_ids(self):
        self.assertEqual(self.status.service_instance_ids('apache2'),
                         ['0a598fda-78b0-47dd-aa21-ef9ec9e6879b'])

    def test_machine_ip(self):
        self.assertEqual(self.status.machine_ip('1'), '10.55.33.6')

    def test_machine_instance_id(self):
        self.assertEqual(self.status.machine_instance_id('1'),
                         '0a598fda-78b0-47dd-aa21-ef9ec9e6879b')
